==> weightjinni_1.5.20080916-1.tar.gz: weightjinni-1.5.20080916/src/Preferences.cpp <==
/*
  This file is part of "WeightJinni" - A program to control your weight.
  Copyright (C) 2008  Tim Teulings

  This library is free software; you can redistribute it and/or
  modify it under the terms of the GNU Lesser General Public
  License as published by the Free Software Foundation; either
  version 2.1 of the License, or (at your option) any later version.

  This library is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  Lesser General Public License for more details.

  You should have received a copy of the GNU Lesser General Public
  License along with this library; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307  USA
*/

#include "Preferences.h"

#include <Lum/Base/L10N.h>

#include <Lum/Button.h>
#include <Lum/ButtonRow.h>
#include <Lum/Combo.h>
#include <Lum/Label.h>
#include <Lum/Panel.h>
#include <Lum/Space.h>
#include <Lum/String.h>
#include <Lum/Tab.h>
#include <Lum/Text.h>
#include <Lum/WindowGroup.h>

#include "Configuration.h"
#include "Util.h"

Preferences::Preferences()
: okAction(new Lum::Model::Action()),

  topWeight(new Lum::Model::String(AmountToString(::topWeight->Get()))),
  bottomWeight(new Lum::Model::String(AmountToString(::bottomWeight->Get()))),
  maxWeight(new Lum::Model::String(AmountToString(::maxWeight->Get()))),
  minWeight(new Lum::Model::String(AmountToString(::minWeight->Get()))),

  topValue(new Lum::Model::String(AmountToString(::topValue->Get()))),
  bottomValue(new Lum::Model::String(AmountToString(::bottomValue->Get()))),
  maxValue(new Lum::Model::String(AmountToString(::maxValue->Get()))),
  minValue(new Lum::Model::String(AmountToString(::minValue->Get()))),
  goodValueLimit(new Lum::Model::String(ValueToString(::goodValueLimit->Get()))),
  badValueLimit(new Lum::Model::String(ValueToString(::badValueLimit->Get()))),
  refValue(new Lum::Model::String(ValueToString(::refValue->Get()))),
  valueUnit(new Lum::Model::String(::valueUnit->Get())),
  weightUnit(new Lum::Model::String(::weightUnit->Get()))
{
  trendType->Push();
  trendDirection->Push();

  AttachModel(okAction);

  AttachModel(topWeight);
  AttachModel(bottomWeight);
  AttachModel(maxWeight);
  AttachModel(minWeight);

  AttachModel(topValue);
  AttachModel(bottomValue);
  AttachModel(maxValue);
  AttachModel(minValue);
  AttachModel(goodValueLimit);
  AttachModel(badValueLimit);

  AttachModel(refValue);

  AttachModel(valueUnit);
  AttachModel(weightUnit);

  SetExitAction(GetClosedAction());
}

Preferences::~Preferences()
{
  trendDirection->Pop();
  trendType->Pop();

  UnattachModel(weightUnit);
  UnattachModel(valueUnit);

  UnattachModel(refValue);

  UnattachModel(minValue);
  UnattachModel(maxValue);
  UnattachModel(bottomValue);
  UnattachModel(topValue);
  UnattachModel(goodValueLimit);
  UnattachModel(badValueLimit);

  UnattachModel(minWeight);
  UnattachModel(maxWeight);
  UnattachModel(bottomWeight);
  UnattachModel(topWeight);

  UnattachModel(okAction);
}

void Preferences::PreInit()
{
  Lum::Button               *button;
  Lum::ButtonRow            *buttonRow;
  Lum::Combo                *combo;
  Lum::Label                *label;
  Lum::Panel                *panel,*panel2;
  Lum::String               *string;
  Lum::Tab                  *tab;
  Lum::WindowGroup          *windowGroup;
  Lum::Model::ListTableRef  table;

  panel=new Lum::VPanel();

  tab=new Lum::Tab();

  label=new Lum::Label();
  label->SetFlex(true,true);

  string=new Lum::String();
  string->SetMinWidth(Lum::Base::Size::stdCharWidth,10);
  string->SetModel(valueUnit);
  label->AddLabel(_(L"PREFS_LABEL_VALUE_UNIT",L"Value unit:"),string);

  string=new Lum::String();
  string->SetMinWidth(Lum::Base::Size::stdCharWidth,10);
  string->SetModel(weightUnit);
  label->AddLabel(_(L"PREFS_LABEL_WEIGHT_UNIT",L"Weight unit:"),string);

  tab->Add(_(L"PREFS_LABEL_UNITS",L"Units"),label);

  label=new Lum::Label();
  label->SetFlex(true,true);

  panel2=new Lum::HPanel();

  string=new Lum::String();
  string->SetMinWidth(Lum::Base::Size::stdCharWidth,5);
  string->SetAlignment(Lum::String::right);
  string->SetModel(bottomWeight);
  panel2->Add(string);

  panel2->Add(new Lum::HSpace());
  panel2->Add(new Lum::Text(L"-"));
  panel2->Add(new Lum::HSpace());

  string=new Lum::String();
  string->SetMinWidth(Lum::Base::Size::stdCharWidth,5);
  string->SetAlignment(Lum::String::right);
  string->SetModel(topWeight);
  panel2->Add(string);

  label->AddLabel(_(L"PREFS_LABEL_WEIGHT_GRAPH",L"Shown weight interval:"),panel2);

  panel2=new Lum::HPanel();

  string=new Lum::String();
  string->SetMinWidth(Lum::Base::Size::stdCharWidth,5);
  string->SetAlignment(Lum::String::right);
  string->SetModel(minWeight);
  panel2->Add(string);

  panel2->Add(new Lum::HSpace());
  panel2->Add(new Lum::Text(L"-"));
  panel2->Add(new Lum::HSpace());

  string=new Lum::String();
  string->SetMinWidth(Lum::Base::Size::stdCharWidth,5);
  string->SetAlignment(Lum::String::right);
  string->SetModel(maxWeight);
  panel2->Add(string);
  label->AddLabel(_(L"PREFS_LABEL_WEIGHT_LIMIT",L"Target range for weight:"),panel2);

  tab->Add(_(L"PREFS_LABEL_WEIGHTS",L"Weights"),label);

  label=new Lum::Label();
  label->SetFlex(true,true);

  panel2=new Lum::HPanel();

  string=new Lum::String();
  string->SetMinWidth(Lum::Base::Size::stdCharWidth,4);
  string->SetAlignment(Lum::String::right);
  string->SetModel(bottomValue);
  panel2->Add(string);

  panel2->Add(new Lum::HSpace());
  panel2->Add(new Lum::Text(L"-"));
  panel2->Add(new Lum::HSpace());

  string=new Lum::String();
  string->SetMinWidth(Lum::Base::Size::stdCharWidth,4);
  string->SetAlignment(Lum::String::right);
  string->SetModel(topValue);
  panel2->Add(string);

  label->AddLabel(_(L"PREFS_LABEL_VALUE_GRAPH",L"Shown value interval:"),panel2);

  panel2=new Lum::HPanel();

  string=new Lum::String();
  string->SetMinWidth(Lum::Base::Size::stdCharWidth,4);
  string->SetAlignment(Lum::String::right);
  string->SetModel(minValue);
  panel2->Add(string);

  panel2->Add(new Lum::HSpace());
  panel2->Add(new Lum::Text(L"-"));
  panel2->Add(new Lum::HSpace());

  string=new Lum::String();
  string->SetMinWidth(Lum::Base::Size::stdCharWidth,4);
  string->SetAlignment(Lum::String::right);
  string->SetModel(maxValue);
  panel2->Add(string);

  label->AddLabel(_(L"PREFS_LABEL_VALUE_LIMIT",L"Target range for values:"),panel2);

  string=new Lum::String();
  string->SetMinWidth(Lum::Base::Size::stdCharWidth,4);
  string->SetAlignment(Lum::String::right);
  string->SetModel(goodValueLimit);
  label->AddLabel(_(L"PREFS_LABEL_GOOD_VALUE_LIMIT",L"Limit for good values:"),string);

  string=new Lum::String();
  string->SetMinWidth(Lum::Base::Size::stdCharWidth,4);
  string->SetAlignment(Lum::String::right);
  string->SetModel(badValueLimit);
  label->AddLabel(_(L"PREFS_LABEL_BAD_VALUE_LIMIT",L"Limit for bad values:"),string);

  tab->Add(_(L"PREFS_LABEL_VALUES",L"Values"),label);

  label=new Lum::Label();
  label->SetFlex(true,true);

  string=new Lum::String();
  string->SetMinWidth(Lum::Base::Size::stdCharWidth,4);
  string->SetAlignment(Lum::String::right);
  string->SetModel(refValue);
  label->AddLabel(_(L"PREFS_LABEL_REF_VALUE",L"Referenz for trend:"),string);

  table=new Lum::Model::ListTable(1);
  table->AppendString(_(L"PREFS_LABEL_TREND_TYPE_REFREL",L"Relative to reference"));
  table->AppendString(_(L"PREFS_LABEL_TREND_TYPE_TWOWEEK",L"2 week trend"));

  combo=new Lum::IndexCombo();
  combo->SetModel(trendType);
  combo->SetTableModel(table);
  label->AddLabel(_(L"PREFS_LABEL_TREND_TYPE",L"Trend type:"),combo);

  table=new Lum::Model::ListTable(1);
  table->AppendString(_(L"PREFS_LABEL_TREND_DIR_DOWN",L"Lower is better"));
  table->AppendString(_(L"PREFS_LABEL_TREND_DIR_UP",L"Higher is better"));

  combo=new Lum::IndexCombo();
  combo->SetModel(trendDirection);
  combo->SetTableModel(table);
  label->AddLabel(_(L"PREFS_LABEL_TREND_DIR",L"Trend direction:"),combo);

  tab->Add(_(L"PREFS_LABEL_TREND",L"Trend"),label);

  panel->Add(tab);

  panel->Add(new Lum::VSpace());

  buttonRow=new Lum::ButtonRow();
  buttonRow->SetFlex(true,false);

  button=new Lum::Button();
  button->SetFlex(true,true);
  button->SetModel(okAction);
  button->SetType(Lum::Button::typeCommit);
  button->SetText(_ld(dlgButtonOk));
  buttonRow->Add(button);

  button=new Lum::Button();
  button->SetFlex(true,true);
  button->SetModel(GetClosedAction());
  button->SetType(Lum::Button::typeCancel);
  button->SetText(_ld(dlgButtonCancel));
  buttonRow->Add(button);

  panel->Add(buttonRow);

  windowGroup=new Lum::WindowGroup();
  windowGroup->SetMain(panel);

  SetTop(windowGroup);

  Dialog::PreInit();
}

void Preferences::Resync(Lum::Base::Model* model, const Lum::Base::ResyncMsg& msg)
{
  if (model==topWeight ||
      model==bottomWeight ||
      model==maxWeight ||
      model==minWeight ||
      model==topValue ||
      model==bottomValue ||
      model==maxValue ||
      model==minValue ||
      model==goodValueLimit ||
      model==badValueLimit) {
    size_t topW,bottomW,topV,bottomV,gVL,bVL,tmp;

    if (StringToAmount(topWeight->Get(),topW) &&
        StringToAmount(bottomWeight->Get(),bottomW) &&
        StringToAmount(maxWeight->Get(),tmp) &&
        StringToAmount(minWeight->Get(),tmp) &&
        StringToAmount(topValue->Get(),topV) &&
        StringToAmount(bottomValue->Get(),bottomV) &&
        StringToAmount(maxValue->Get(),tmp) &&
        StringToAmount(minValue->Get(),tmp) &&
        StringToAmount(goodValueLimit->Get(),gVL) &&
        StringToAmount(badValueLimit->Get(),bVL) &&
        StringToAmount(refValue->Get(),tmp) &&
        topW>bottomW &&
        topV>bottomV &&
        gVL<bVL) {
      okAction->Enable();
    }
    else {
      okAction->Disable();
    }
  }
  else if (model==okAction && okAction->IsFinished()) {
    size_t topW,bottomW,maxW,minW,topV,bottomV,maxV,minV,goodVL,badVL,refV;

    assert(StringToAmount(topWeight->Get(),topW) &&
           StringToAmount(bottomWeight->Get(),bottomW) &&
           StringToAmount(maxWeight->Get(),maxW) &&
           StringToAmount(minWeight->Get(),minW) &&
           StringToAmount(topValue->Get(),topV) &&
           StringToAmount(bottomValue->Get(),bottomV) &&
           StringToAmount(maxValue->Get(),maxV) &&
           StringToAmount(minValue->Get(),minV) &&
           StringToAmount(goodValueLimit->Get(),goodVL) &&
           StringToAmount(badValueLimit->Get(),badVL) &&
           StringToAmount(refValue->Get(),refV));

    ::valueUnit->Set(valueUnit->Get());
    ::weightUnit->Set(weightUnit->Get());

    ::topWeight->Set(topW);
    ::bottomWeight->Set(bottomW);
    ::maxWeight->Set(maxW);
    ::minWeight->Set(minW);

    ::topValue->Set(topV);
    ::bottomValue->Set(bottomV);
    ::maxValue->Set(maxV);
    ::minValue->Set(minV);
    ::goodValueLimit->Set(goodVL);
    ::badValueLimit->Set(badVL);

    ::refValue->Set(refV);

    trendType->Save();
    trendDirection->Save();

    Exit();
  }

  Dialog::Resync(model,msg);
}

==> weightjinni_1.5.20081120-1.tar.gz: weightjinni-1.5.20081120/src/Preferences.cpp <==
/*
  This file is part of "WeightJinni" - A program to control your weight.
  Copyright (C) 2008  Tim Teulings

  This library is free software; you can redistribute it and/or
  modify it under the terms of the GNU Lesser General Public
  License as published by the Free Software Foundation; either
  version 2.1 of the License, or (at your option) any later version.

  This library is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  Lesser General Public License for more details.

  You should have received a copy of the GNU Lesser General Public
  License along with this library; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307  USA
*/

#include "Preferences.h"

#include <Lum/Base/L10N.h>

#include <Lum/ButtonRow.h>
#include <Lum/Combo.h>
#include <Lum/Label.h>
#include <Lum/Panel.h>
#include <Lum/Space.h>
#include <Lum/String.h>
#include <Lum/Tab.h>
#include <Lum/Text.h>

#include "Configuration.h"
#include "Util.h"

Preferences::Preferences()
: okAction(new Lum::Model::Action()),

  topWeight(new Lum::Model::String(AmountToString(::topWeight->Get()))),
  bottomWeight(new Lum::Model::String(AmountToString(::bottomWeight->Get()))),
  maxWeight(new Lum::Model::String(AmountToString(::maxWeight->Get()))),
  minWeight(new Lum::Model::String(AmountToString(::minWeight->Get()))),

  topValue(new Lum::Model::String(AmountToString(::topValue->Get()))),
  bottomValue(new Lum::Model::String(AmountToString(::bottomValue->Get()))),
  maxValue(new Lum::Model::String(AmountToString(::maxValue->Get()))),
  minValue(new Lum::Model::String(AmountToString(::minValue->Get()))),
  goodValueLimit(new Lum::Model::String(ValueToString(::goodValueLimit->Get()))),
  badValueLimit(new Lum::Model::String(ValueToString(::badValueLimit->Get()))),
  refValue(new Lum::Model::String(ValueToString(::refValue->Get()))),
  valueUnit(new Lum::Model::String(::valueUnit->Get())),
  weightUnit(new Lum::Model::String(::weightUnit->Get()))
{
  trendType->Push();
  trendDirection->Push();

  AttachModel(okAction);

  AttachModel(topWeight);
  AttachModel(bottomWeight);
  AttachModel(maxWeight);
  AttachModel(minWeight);

  AttachModel(topValue);
  AttachModel(bottomValue);
  AttachModel(maxValue);
  AttachModel(minValue);
  AttachModel(goodValueLimit);
  AttachModel(badValueLimit);

  AttachModel(refValue);

  AttachModel(valueUnit);
  AttachModel(weightUnit);

  SetExitAction(GetClosedAction());
}

Preferences::~Preferences()
{
  trendDirection->Pop();
  trendType->Pop();
}

void Preferences::PreInit()
{
  Lum::ButtonRow            *buttonRow;
  Lum::Combo                *combo;
  Lum::Label                *label;
  Lum::Panel                *panel,*panel2;
  Lum::String               *string;
  Lum::Tab                  *tab;
  Lum::Model::ListTableRef  table;

  panel=new Lum::VPanel();

  tab=new Lum::Tab();

  label=new Lum::Label();
  label->SetFlex(true,true);

  string=new Lum::String();
  string->SetMinWidth(Lum::Base::Size::stdCharWidth,10);
  string->SetModel(valueUnit);
  label->AddLabel(_(L"PREFS_LABEL_VALUE_UNIT",L"Value unit:"),string);

  string=new Lum::String();
  string->SetMinWidth(Lum::Base::Size::stdCharWidth,10);
  string->SetModel(weightUnit);
  label->AddLabel(_(L"PREFS_LABEL_WEIGHT_UNIT",L"Weight unit:"),string);

  tab->Add(_(L"PREFS_LABEL_UNITS",L"Units"),label);

  label=new Lum::Label();
  label->SetFlex(true,true);

  panel2=new Lum::HPanel();

  string=new Lum::String();
  string->SetMinWidth(Lum::Base::Size::stdCharWidth,5);
  string->SetAlignment(Lum::String::right);
  string->SetInputHelper(new Lum::InputHelperNumber(Lum::InputHelperNumber::typeFloat));
  string->SetModel(bottomWeight);
  panel2->Add(string);

  panel2->Add(new Lum::HSpace());
  panel2->Add(new Lum::Text(L"-"));
  panel2->Add(new Lum::HSpace());

  string=new Lum::String();
  string->SetMinWidth(Lum::Base::Size::stdCharWidth,5);
  string->SetAlignment(Lum::String::right);
  string->SetInputHelper(new Lum::InputHelperNumber(Lum::InputHelperNumber::typeFloat));
  string->SetModel(topWeight);
  panel2->Add(string);

  label->AddLabel(_(L"PREFS_LABEL_WEIGHT_GRAPH",L"Shown weight interval:"),panel2);

  panel2=new Lum::HPanel();

  string=new Lum::String();
  string->SetMinWidth(Lum::Base::Size::stdCharWidth,5);
  string->SetAlignment(Lum::String::right);
  string->SetInputHelper(new Lum::InputHelperNumber(Lum::InputHelperNumber::typeFloat));
  string->SetModel(minWeight);
  panel2->Add(string);

  panel2->Add(new Lum::HSpace());
  panel2->Add(new Lum::Text(L"-"));
  panel2->Add(new Lum::HSpace());

  string=new Lum::String();
  string->SetMinWidth(Lum::Base::Size::stdCharWidth,5);
  string->SetAlignment(Lum::String::right);
  string->SetInputHelper(new Lum::InputHelperNumber(Lum::InputHelperNumber::typeFloat));
  string->SetModel(maxWeight);
  panel2->Add(string);
  label->AddLabel(_(L"PREFS_LABEL_WEIGHT_LIMIT",L"Target range for weight:"),panel2);

  tab->Add(_(L"PREFS_LABEL_WEIGHTS",L"Weights"),label);

  label=new Lum::Label();
  label->SetFlex(true,true);

  panel2=new Lum::HPanel();

  string=new Lum::String();
  string->SetMinWidth(Lum::Base::Size::stdCharWidth,4);
  string->SetAlignment(Lum::String::right);
  string->SetInputHelper(new Lum::InputHelperNumber(Lum::InputHelperNumber::typeFloat));
  string->SetModel(bottomValue);
  panel2->Add(string);

  panel2->Add(new Lum::HSpace());
  panel2->Add(new Lum::Text(L"-"));
  panel2->Add(new Lum::HSpace());

  string=new Lum::String();
  string->SetMinWidth(Lum::Base::Size::stdCharWidth,4);
  string->SetAlignment(Lum::String::right);
  string->SetInputHelper(new Lum::InputHelperNumber(Lum::InputHelperNumber::typeFloat));
  string->SetModel(topValue);
  panel2->Add(string);

  label->AddLabel(_(L"PREFS_LABEL_VALUE_GRAPH",L"Shown value interval:"),panel2);

  panel2=new Lum::HPanel();

  string=new Lum::String();
  string->SetMinWidth(Lum::Base::Size::stdCharWidth,4);
  string->SetAlignment(Lum::String::right);
  string->SetInputHelper(new Lum::InputHelperNumber(Lum::InputHelperNumber::typeFloat));
  string->SetModel(minValue);
  panel2->Add(string);

  panel2->Add(new Lum::HSpace());
  panel2->Add(new Lum::Text(L"-"));
  panel2->Add(new Lum::HSpace());

  string=new Lum::String();
  string->SetMinWidth(Lum::Base::Size::stdCharWidth,4);
  string->SetAlignment(Lum::String::right);
  string->SetInputHelper(new Lum::InputHelperNumber(Lum::InputHelperNumber::typeFloat));
  string->SetModel(maxValue);
  panel2->Add(string);

  label->AddLabel(_(L"PREFS_LABEL_VALUE_LIMIT",L"Target range for values:"),panel2);

  string=new Lum::String();
  string->SetMinWidth(Lum::Base::Size::stdCharWidth,4);
  string->SetAlignment(Lum::String::right);
  string->SetInputHelper(new Lum::InputHelperNumber(Lum::InputHelperNumber::typeFloat));
  string->SetModel(goodValueLimit);
  label->AddLabel(_(L"PREFS_LABEL_GOOD_VALUE_LIMIT",L"Limit for good values:"),string);

  string=new Lum::String();
  string->SetMinWidth(Lum::Base::Size::stdCharWidth,4);
  string->SetAlignment(Lum::String::right);
  string->SetInputHelper(new Lum::InputHelperNumber(Lum::InputHelperNumber::typeFloat));
  string->SetModel(badValueLimit);
  label->AddLabel(_(L"PREFS_LABEL_BAD_VALUE_LIMIT",L"Limit for bad values:"),string);

  tab->Add(_(L"PREFS_LABEL_VALUES",L"Values"),label);

  label=new Lum::Label();
  label->SetFlex(true,true);

  string=new Lum::String();
  string->SetMinWidth(Lum::Base::Size::stdCharWidth,4);
  string->SetAlignment(Lum::String::right);
  string->SetInputHelper(new Lum::InputHelperNumber(Lum::InputHelperNumber::typeFloat));
  string->SetModel(refValue);
  label->AddLabel(_(L"PREFS_LABEL_REF_VALUE",L"Reference for trend:"),string);

  table=new Lum::Model::ListTable(1);
  table->AppendString(_(L"PREFS_LABEL_TREND_TYPE_REFREL",L"Relative to reference"));
  table->AppendString(_(L"PREFS_LABEL_TREND_TYPE_TWOWEEK",L"2 week trend"));

  combo=new Lum::IndexCombo();
  combo->SetModel(trendType);
  combo->SetTableModel(table);
  label->AddLabel(_(L"PREFS_LABEL_TREND_TYPE",L"Trend type:"),combo);

  table=new Lum::Model::ListTable(1);
  table->AppendString(_(L"PREFS_LABEL_TREND_DIR_DOWN",L"Lower is better"));
  table->AppendString(_(L"PREFS_LABEL_TREND_DIR_UP",L"Higher is better"));

  combo=new Lum::IndexCombo();
  combo->SetModel(trendDirection);
  combo->SetTableModel(table);
  label->AddLabel(_(L"PREFS_LABEL_TREND_DIR",L"Trend direction:"),combo);

  tab->Add(_(L"PREFS_LABEL_TREND",L"Trend"),label);

  panel->Add(tab);

  panel->Add(new Lum::VSpace());

  buttonRow=Lum::ButtonRow::CreateOkCancel(okAction,GetClosedAction());
  buttonRow->SetFlex(true,false);
  panel->Add(buttonRow);

  SetMain(panel);


  Dialog::PreInit();
}

void Preferences::Resync(Lum::Base::Model* model, const Lum::Base::ResyncMsg& msg)
{
  if (model==topWeight ||
      model==bottomWeight ||
      model==maxWeight ||
      model==minWeight ||
      model==topValue ||
      model==bottomValue ||
      model==maxValue ||
      model==minValue ||
      model==goodValueLimit ||
      model==badValueLimit) {
    size_t topW,bottomW,topV,bottomV,gVL,bVL,tmp;

    if (StringToAmount(topWeight->Get(),topW) &&
        StringToAmount(bottomWeight->Get(),bottomW) &&
        StringToAmount(maxWeight->Get(),tmp) &&
        StringToAmount(minWeight->Get(),tmp) &&
        StringToAmount(topValue->Get(),topV) &&
        StringToAmount(bottomValue->Get(),bottomV) &&
        StringToAmount(maxValue->Get(),tmp) &&
        StringToAmount(minValue->Get(),tmp) &&
        StringToAmount(goodValueLimit->Get(),gVL) &&
        StringToAmount(badValueLimit->Get(),bVL) &&
        StringToAmount(refValue->Get(),tmp) &&
        topW>bottomW &&
        topV>bottomV &&
        gVL<bVL) {
      okAction->Enable();
    }
    else {
      okAction->Disable();
    }
  }
  else if (model==okAction && okAction->IsFinished()) {
    size_t topW,bottomW,maxW,minW,topV,bottomV,maxV,minV,goodVL,badVL,refV;

    assert(StringToAmount(topWeight->Get(),topW) &&
           StringToAmount(bottomWeight->Get(),bottomW) &&
           StringToAmount(maxWeight->Get(),maxW) &&
           StringToAmount(minWeight->Get(),minW) &&
           StringToAmount(topValue->Get(),topV) &&
           StringToAmount(bottomValue->Get(),bottomV) &&
           StringToAmount(maxValue->Get(),maxV) &&
           StringToAmount(minValue->Get(),minV) &&
           StringToAmount(goodValueLimit->Get(),goodVL) &&
           StringToAmount(badValueLimit->Get(),badVL) &&
           StringToAmount(refValue->Get(),refV));

    ::valueUnit->Set(valueUnit->Get());
    ::weightUnit->Set(weightUnit->Get());

    ::topWeight->Set(topW);
    ::bottomWeight->Set(bottomW);
    ::maxWeight->Set(maxW);
    ::minWeight->Set(minW);

    ::topValue->Set(topV);
    ::bottomValue->Set(bottomV);
    ::maxValue->Set(maxV);
    ::minValue->Set(minV);
    ::goodValueLimit->Set(goodVL);
    ::badValueLimit->Set(badVL);

    ::refValue->Set(refV);

    trendType->Save();
    trendDirection->Save();

    Exit();
  }

  Dialog::Resync(model,msg);
}
